package cn.com.infosec.crypto.signers;

import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DSA;
import cn.com.infosec.crypto.params.DSAKeyParameters;
import cn.com.infosec.crypto.params.DSAParameters;
import cn.com.infosec.crypto.params.DSAPrivateKeyParameters;
import cn.com.infosec.crypto.params.DSAPublicKeyParameters;
import cn.com.infosec.crypto.params.ParametersWithRandom;
import java.math.BigInteger;
import java.security.SecureRandom;

public class DSASigner implements DSA {
   DSAKeyParameters key;
   SecureRandom random;

   private BigInteger calculateE(BigInteger n, byte[] message) {
      if (n.bitLength() >= message.length * 8) {
         return new BigInteger(1, message);
      } else {
         int var3;
         byte[] n1 = new byte[var3 = n.bitLength() / 8];
         System.arraycopy(message, 0, n1, 0, var3);
         return new BigInteger(1, n1);
      }
   }

   public void init(boolean forSigning, CipherParameters param) {
      if (forSigning) {
         if (param instanceof ParametersWithRandom) {
            DSASigner var10000 = this;
            ParametersWithRandom var3;
            this.random = (var3 = (ParametersWithRandom)param).getRandom();
            var10000.key = (DSAPrivateKeyParameters)var3.getParameters();
         } else {
            this.random = new SecureRandom();
            this.key = (DSAPrivateKeyParameters)param;
         }
      } else {
         this.key = (DSAPublicKeyParameters)param;
      }

   }

   public BigInteger[] generateSignature(byte[] message) {
      DSAParameters var2;
      BigInteger message1 = this.calculateE((var2 = this.key.getParameters()).getQ(), message);
      int var3 = var2.getQ().bitLength();

      BigInteger var10000;
      BigInteger var4;
      do {
         var10000 = var4 = new BigInteger;
         var4.<init>(var3, this.random);
      } while(var10000.compareTo(var2.getQ()) >= 0);

      BigInteger var7 = var2.getG().modPow(var4, var2.getP()).mod(var2.getQ());
      BigInteger var5 = var4.modInverse(var2.getQ()).multiply(message1.add(((DSAPrivateKeyParameters)this.key).getX().multiply(var7))).mod(var2.getQ());
      return new BigInteger[]{var7, var5};
   }

   public boolean verifySignature(byte[] message, BigInteger r, BigInteger s) {
      DSAParameters var4;
      BigInteger message1 = this.calculateE((var4 = this.key.getParameters()).getQ(), message);
      BigInteger var5;
      if ((var5 = BigInteger.valueOf(0L)).compareTo(r) < 0 && var4.getQ().compareTo(r) > 0) {
         if (var5.compareTo(s) < 0 && var4.getQ().compareTo(s) > 0) {
            s = message1.multiply(message1 = s.modInverse(var4.getQ())).mod(var4.getQ());
            message1 = r.multiply(message1).mod(var4.getQ());
            return var4.getG().modPow(s, var4.getP()).multiply(((DSAPublicKeyParameters)this.key).getY().modPow(message1, var4.getP())).mod(var4.getP()).mod(var4.getQ()).equals(r);
         } else {
            return false;
         }
      } else {
         return false;
      }
   }
}
